@@ -23,6 +23,10 @@
 
 #ifdef WEBP_HAVE_GIF
 
+#if defined(HAVE_UNISTD_H) && HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <gif_lib.h>
 #include "webp/encode.h"
 #include "webp/mux.h"
@@ -30,6 +34,10 @@
 #include "../imageio/imageio_util.h"
 #include "./gifdec.h"
 
+#if !defined(STDIN_FILENO)
+#define STDIN_FILENO 0
+#endif
+
 //------------------------------------------------------------------------------
 
 static int transparent_index = GIF_INDEX_INVALID;  // Opaque by default.
@@ -92,7 +100,6 @@ int main(int argc, const char *argv[]) {
   WebPMuxError err = WEBP_MUX_OK;
   int ok = 0;
   const char *in_file = NULL, *out_file = NULL;
-  FILE* out = NULL;
   GifFileType* gif = NULL;
   int frame_duration = 0;
   int frame_timestamp = 0;
@@ -263,9 +270,11 @@ int main(int argc, const char *argv[]) {
 
   // Start the decoder object
 #if LOCAL_GIF_PREREQ(5,0)
-  gif = DGifOpenFileName(in_file, &gif_error);
+  gif = !strcmp(in_file, "-") ? DGifOpenFileHandle(STDIN_FILENO, &gif_error)
+                              : DGifOpenFileName(in_file, &gif_error);
 #else
-  gif = DGifOpenFileName(in_file);
+  gif = !strcmp(in_file, "-") ? DGifOpenFileHandle(STDIN_FILENO)
+                              : DGifOpenFileName(in_file);
 #endif
   if (gif == NULL) goto End;
 
@@ -351,6 +360,14 @@ int main(int argc, const char *argv[]) {
         GIFDisposeFrame(orig_dispose, &gif_rect, &prev_canvas, &curr_canvas);
         GIFCopyPixels(&curr_canvas, &prev_canvas);
 
+        // Force frames with a small or no duration to 100ms to be consistent
+        // with web browsers and other transcoding tools. This also avoids
+        // incorrect durations between frames when padding frames are
+        // discarded.
+        if (frame_duration <= 10) {
+          frame_duration = 100;
+        }
+
         // Update timestamp (for next frame).
         frame_timestamp += frame_duration;
 
@@ -460,7 +477,7 @@ int main(int argc, const char *argv[]) {
         stored_loop_count = 1;
         loop_count = 1;
       }
-    } else if (loop_count > 0) {
+    } else if (loop_count > 0 && loop_count < 65535) {
       // adapt GIF's semantic to WebP's (except in the infinite-loop case)
       loop_count += 1;
     }
@@ -532,8 +549,13 @@ int main(int argc, const char *argv[]) {
       goto End;
     }
     if (!quiet) {
-      fprintf(stderr, "Saved output file (%d bytes): %s\n",
-              (int)webp_data.size, out_file);
+      if (!strcmp(out_file, "-")) {
+        fprintf(stderr, "Saved %d bytes to STDIO\n",
+                (int)webp_data.size);
+      } else {
+        fprintf(stderr, "Saved output file (%d bytes): %s\n",
+                (int)webp_data.size, out_file);
+      }
     }
   } else {
     if (!quiet) {
@@ -555,7 +577,6 @@ int main(int argc, const char *argv[]) {
   WebPPictureFree(&curr_canvas);
   WebPPictureFree(&prev_canvas);
   WebPAnimEncoderDelete(enc);
-  if (out != NULL && out_file != NULL) fclose(out);
 
   if (gif_error != GIF_OK) {
     GIFDisplayError(gif, gif_error);
